/// <reference types="react" />
import { List } from 'immutable';
import { ListenerFn } from 'eventemitter3';
import * as React from 'react';
import { ReactNode } from 'react';
import { Record } from 'immutable';

/**
 * @internal
 */
export declare function addServerUrlToResourceProps(props: {
    [key: string]: any;
}): {
    [key: string]: any;
};

/**
 * @public
 */
export declare function Animatable<Value>(value: Value | Animatable<Value>): Animatable<Value>;

/**
 * @public
 */
export declare interface Animatable<Value> extends UpdateObserver<Value> {
    get(): Value;
    set(value: Value | Animatable<Value>): void;
    set(value: Value | Animatable<Value>, transaction?: TransactionId): void;
    /**
     * @alpha
     */
    finishTransaction(transaction: TransactionId): FinishFunction[];
}

/**
 * @public
 */
export declare namespace Animatable {
    /**
     * @alpha
     */
    export function transaction(update: (updater: (animatable: Animatable<any>, value: any) => void, transactionId: TransactionId) => void): void;
    /**
     * @public
     */
    export function getNumber(value: number | Animatable<number> | null | undefined, defaultValue?: number): number;
    /**
     * @public
     */
    export function get<Value>(value: Value | Animatable<Value> | null | undefined, defaultValue: Value): Value;
    /**
     * @internal
     */
    export function objectToValues<Object>(object: AnimatableObject<Object>): Object;
}

/** @public */
export declare type AnimatableObject<T> = {
    [K in keyof T]: ToAnimatableOrValue<T[K]>;
};

/**
 * Animate a value or object.
 *
 * @remarks
 * Recommended use is to use convenience functions from the `animate` namespace
 * instead of passing an animator. Only use this for low-level animation tweaking.
 *
 * @param from - The animatable value or object to start from
 * @param to - The value to animate to
 * @param animator - Animator to use.
 * @param options - Animation options
 * @public
 */
export declare function animate<Value, Options>(from: Animatable<Value> | AnimatableObject<Value>, to: Value, animator?: AnimatorClass<Value, Options>, options?: Partial<Options & AnimationOptions<Value>>): FramerAnimation<Value, Options>;

/**
 * @public
 */
export declare namespace animate {
    export function spring<Value>(from: Animatable<Value> | AnimatableObject<Value>, to: Value, options?: Partial<SpringOptions & AnimationOptions<Value>>): FramerAnimation<Value, SpringOptions>;
    export function bezier<Value>(from: Animatable<Value> | AnimatableObject<Value>, to: Value, options?: Partial<BezierOptions & AnimationOptions<Value>>): FramerAnimation<Value, BezierOptions>;
    export function linear<Value>(from: Animatable<Value> | AnimatableObject<Value>, to: Value, options?: Partial<EaseOptions & AnimationOptions<Value>>): FramerAnimation<Value, BezierOptions>;
    export function ease<Value>(from: Animatable<Value> | AnimatableObject<Value>, to: Value, options?: Partial<EaseOptions & AnimationOptions<Value>>): FramerAnimation<Value, BezierOptions>;
    export function easeIn<Value>(from: Animatable<Value> | AnimatableObject<Value>, to: Value, options?: Partial<EaseOptions & AnimationOptions<Value>>): FramerAnimation<Value, BezierOptions>;
    export function easeOut<Value>(from: Animatable<Value> | AnimatableObject<Value>, to: Value, options?: Partial<EaseOptions & AnimationOptions<Value>>): FramerAnimation<Value, BezierOptions>;
    export function easeInOut<Value>(from: Animatable<Value> | AnimatableObject<Value>, to: Value, options?: Partial<EaseOptions & AnimationOptions<Value>>): FramerAnimation<Value, BezierOptions>;
}

/**
 * @internal
 */
declare abstract class AnimationDriver<AnimatorClass extends Animator<Value, Options>, Value, Options> implements AnimationInterface {
    animator: AnimatorClass;
    protected updateCallback: (value: Value) => void;
    protected finishedCallback?: ((isFinished: boolean) => void) | undefined;
    constructor(animator: AnimatorClass, updateCallback: (value: Value) => void, finishedCallback?: ((isFinished: boolean) => void) | undefined);
    abstract play(): void;
    protected update: (frame: number, elapsed: number) => void;
    abstract cancel(): void;
    finish(): void;
    isFinished(): boolean;
}

/**
 * @public
 */
declare interface AnimationInterface {
    /**
     * @internal
     */
    play(): void;
    cancel(): void;
    /**
     * @internal
     */
    finish(): void;
    isFinished(): boolean;
}

declare interface AnimationOptions<Value> extends InterpolationOptions {
    /**
     * @beta
     */
    customInterpolation?: Interpolation<Value>;
    /**
     * @beta
     */
    precalculate: boolean;
}

/**
 * @beta
 */
declare interface Animator<Value, Options = any> {
    /**
     * @beta
     */
    setFrom(from: Value): void;
    /**
     * @beta
     */
    setTo(to: Value): void;
    /**
     * @beta
     */
    isReady(): boolean;
    /**
     * @beta
     */
    next(delta: number): Value;
    /**
     * @beta
     */
    isFinished(): boolean;
}

/**
 * @public
 */
declare interface AnimatorClass<Value, Options = any> {
    /**
     * @beta
     */
    new (options: Partial<Options>, interpolation: Interpolation<Value>): Animator<Value, Options>;
}

/**
 * @internal
 */
export declare const AnyInterpolation: ValueInterpolation;

/** @public */
export declare interface ArrayControlDescription<P = any, Q = any> extends BaseControlDescription<P> {
    type: ControlType.Array;
    propertyControl: FlatControlDescription<Q>;
    maxCount?: number;
    defaultValue?: any[];
}

/**
 * @beta
 */
declare type Axis = "x" | "y";

declare type Background = Color | Gradient | BackgroundImage | string;

declare interface BackgroundFilterProperties {
    backgroundBlur: number;
}

declare interface BackgroundImage {
    src: string;
    pixelWidth?: number;
    pixelHeight?: number;
    intrinsicWidth?: number;
    intrinsicHeight?: number;
    fit?: ImageFit;
}

declare namespace BackgroundImage {
    const isImageObject: (image: any) => image is object & BackgroundImage;
}

declare interface BackgroundProperties {
    background: Animatable<Background> | Background | null;
}

declare interface BaseControlDescription<P = any> {
    title?: string;
    hidden?(props: P): boolean;
}

declare enum Bezier {
    Linear = "linear",
    Ease = "ease",
    EaseIn = "ease-in",
    EaseOut = "ease-out",
    EaseInOut = "ease-in-out"
}

/**
 * Animator class using a bezier curve.
 * @beta
 */
export declare class BezierAnimator<Value> implements Animator<Value, BezierOptions> {
    private interpolation;
    private unitBezier;
    private options;
    current: Value;
    destination: Value;
    interpolator: Interpolator<Value>;
    progress: number;
    constructor(options: Partial<BezierOptions>, interpolation: Interpolation<Value>);
    setFrom(value: Value): void;
    setTo(value: Value): void;
    isReady(): boolean;
    updateInterpolator(): void;
    next: (delta: number) => Value;
    isFinished(): boolean;
    solveEpsilon(duration: number): number;
}

declare interface BezierOptions {
    curve: Curve;
    duration: number;
}

declare type BlendingMode = "normal" | "multiply" | "screen" | "overlay" | "darken" | "lighten" | "color-dodge" | "color-burn" | "hard-light" | "soft-light" | "difference" | "exclusion" | "hue" | "saturation" | "color" | "luminosity";

declare interface BlendingProperties {
    blendingMode: BlendingMode;
}

declare interface BooleanControlDescription<P = any> extends BaseControlDescription<P> {
    type: ControlType.Boolean;
    defaultValue?: boolean;
    disabledTitle?: string;
    enabledTitle?: string;
}

declare type BorderStyle = "solid" | "dashed" | "dotted" | "double";

declare interface BoxShadow {
    inset: boolean;
    color: string;
    x: number;
    y: number;
    blur: number;
    spread: number;
}

declare namespace BoxShadow {
    function is(shadow: any): shadow is BoxShadow;
    function toCSS(shadow: BoxShadow): string;
}

declare interface BoxShadowProperties {
    shadows: BoxShadow[];
}

/** @internal */
export declare const calcChildLayoutRects: (childSizes: Size[], size: Size, props: StackProperties, invisibleItems: number[]) => Rect[];

/** @public */
export declare type Cancel = () => void;

/**
 * @internal
 */
export declare class CanvasStore {
    canvas: PropertyTree;
    listeners: React.Component[];
    ids: string[];
    static __shared: CanvasStore | null;
    static shared(data?: PropertyTree): CanvasStore;
    updateNode(presentationNode: PropertyTree): void;
    setCanvas(canvas: PropertyTree): void;
    registerListener(listener: React.Component, idOrName: string): PropertyTree | null;
    removeListener(listener: React.Component): void;
}

declare interface Change<Value> {
    value: Value;
    oldValue?: Value;
}

/**
 * The Color function can be used to define colors, either as a string value or as an object. All colors
 * are converted to a Color object with `r, g, b`, `h, s, l` and an `a` value.
 * There are also various helpers on the Color function for working with,
 * modifying and detecting colors.
 *
 * ```typescript
 * // HEX
 * const blue = Color("#28AFFA")
 *
 * // RGB
 * const blue = Color("rgb(255, 0, 102)")
 * const blue = Color(255, 0, 102, 255})
 * const blue = Color({r: 255, g: 0, b: 102})
 * const blue = Color({r: 255, g: 0, b: 102, a: 255})
 *
 * // HSL
 * const blue = Color("hsl(201, 95, 57)")
 * const blue = Color({h: 201, s: 95, l: 57})
 * const blue = Color({h: 201, s: 95, l: 57, a: 1})
 * ```
 * @public
 */
export declare function Color(color: IncomingColor | Color | number, r?: number, g?: number, b?: number): Color;

/**
 * @public
 */
export declare interface Color {
    r: number;
    g: number;
    b: number;
    h: number;
    s: number;
    l: number;
    a: number;
    roundA: number;
    format: ColorFormat;
    initialValue?: string;
    isValid?: boolean;
}

/**
 * @public
 */
export declare namespace Color {
    /**
     * Formats a Color object into a readable string for debugging.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * Color.inspect(blue) //=> "<Object r:40 g:175 b:250 a:1>"
     * ```
     *
     * @param color - The Color object to format
     * @param initialValue - A canonical hex string to be used instead of an rgba() value.
     */
    export function inspect(color: Color, initialValue?: string): string;
    /**
     * Checks if the value is a valid color object or color string. Returns true or false.
     *
     * @remarks
     * ```typescript
     * Color.isColor("#28AFFA") //=> true
     * Color.isColor(Color("#28AFFA")) //=> true
     * ```
     *
     * @param color - The potential color value to validate
     */
    export function isColor(color: string | Color): boolean;
    /**
     * Checks if the value is a valid color string. Returns true or false.
     *
     * @remarks
     * ```typescript
     * Color.isColorString("#28AFFA") //=> true
     * ```
     *
     * @param color - A string representing a color
     */
    export function isColorString(colorString: string | object): boolean;
    /**
     * Checks if the value is a valid Color object. Returns true or false.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * Color.isColorObject(blue) //=> true
     * Color.isColorObject("#28AFFA") //=> false
     * ```
     *
     * @param color - An object representing a color.
     */
    export function isColorObject(color: any): color is object & Color;
    /**
     * Formats a Color instance into an RGB string.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * Color.toString(blue) //=> "rgb(40, 175, 250)"
     * ```
     *
     * @param color - The color to format
     */
    export function toString(color: Color): string;
    /**
     * Formats a Color instance into an hexidecimal value.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * Color.toHex(blue) //=> "28affa"
     * Color.toHex(Color("#FFAAFF"), true) //=> "faf"
     * ```
     *
     * @param color - The color to format
     * @param allow3Char - If true will return short hand colors if possible (defaults to false).
     */
    export function toHex(color: Color, allow3Char?: boolean): string;
    /**
     * Formats a Color instance into an hexidecimal string.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * Color.toHexString(blue) //=> "#28affa"
     * Color.toHexString(Color("#FFAAFF"), true) //=> "#faf"
     * ```
     *
     * @param color - The color to format
     * @param allow3Char - If true will return short hand colors if possible (defaults to false).
     */
    export function toHexString(color: Color, allow3Char?: boolean): string;
    /**
     * Formats a Color instance into an RGB string.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * Color.toRgbString(blue) //=> "rgb(40, 175, 250)"
     * ```
     *
     * @param color - The color to format
     */
    export function toRgbString(color: Color): string;
    /**
     * Formats a Color instance into an HUSL object.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * Color.toHusl(blue) //=> {h: 240, s: 95, l: , a: 1}
     * ```
     *
     * @param color - The color to format
     */
    export function toHusl(color: Color): ColorHSLA;
    /**
     * Formats a Color instance into an HSL string.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * Color.toHslString(blue) //=> "hsl(205, 95%, 57%)"
     * ```
     *
     * @param color - The color to format
     */
    export function toHslString(color: Color): string;
    /**
     * Formats a Color instance into an HSV object.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * Color.toHsv(blue) //=> {h: 201, s: 0.84, v: 0.98, a: 1}"
     * ```
     *
     * @param color - The color to format
     */
    export function toHsv(color: Color): ColorHSVA;
    /**
     * Formats a Color instance into an HSV string.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * Color.toHslString(blue) //=> "hsv(201, 84%, 98%)"
     * ```
     *
     * @param color - The color to format
     */
    export function toHsvString(color: Color): string;
    /**
     * Formats a Color instance into {@link https://css-tricks.com/snippets/css/named-colors-and-hex-equivalents/ | CSS name}
     * or returns false if unspecified.
     *
     * @remarks
     * ```typescript
     * const green = Color("#8FBC8F")
     * Color.toName(green) //=> "darkseagreen"
     * ```
     *
     * @param color - The color to format
     */
    export function toName(color: Color): string | false;
    /**
     * Formats a color into an HSL object.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * Color.toHsl(blue) //=> {h: 201, s: 0.95, l: 0.56, a: 1}
     * ```
     *
     * @param color - The color to format
     */
    export function toHsl(color: Color): ColorHSLA;
    /**
     * Formats a color into an RGB object.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * Color.toRgb(blue) //=> {r: 40, g: 175, b: 250, a: 1}
     * ```
     *
     * @param color - The color to format
     */
    export function toRgb(color: Color): ColorRGBA;
    /**
     * Returns a brightened color.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * const brightblue = Color.lighten(blue, 20)
     * ```
     *
     * @param color - The color to brighten
     * @param amount - A number, from 0 to 100. Set to 10 by default.
     */
    export function brighten(color: Color, amount?: number): Color;
    /**
     * Add white and return a lightened color.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * const lightblue = Color.lighten(blue, 20)
     * ```
     *
     * @param color - The color to lighten
     * @param amount - A number, from 0 to 100. Set to 10 by default.
     */
    export function lighten(color: Color, amount?: number): Color;
    /**
     * Add black and return a darkened color.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * const darkblue = Color.darken(blue, 20)
     * ```
     * @param color - The color to darken.
     * @param amount - A number, from 0 to 100. Set to 10 by default.
     */
    export function darken(color: Color, amount?: number): Color;
    /**
     * Increase the saturation of a color.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * const saturated = Color.saturate(blue, 100)
     * ```
     * @param color - The color to modify
     * @param amount - A number from 0 to 100. Set to 10 by default.
     */
    export function saturate(color: Color, amount?: number): Color;
    /**
     * Decrease the saturation of a color.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * const desaturated = Color.desaturate(blue, 100)
     * ```
     * @param color - The color to modify
     * @param amount - A number from 0 to 100. Set to 10 by default.
     */
    export function desaturate(color: Color, amount?: number): Color;
    /**
     * Return a fully desaturated color.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * const gray = Color.grayscale(blue)
     * ```
     * @param color - The color to convert.
     */
    export function grayscale(color: Color): Color;
    /**
     * Returns a new color for the rotated hue.
     * @param color The color to manipulate
     * @param angle The angle in degrees in which to rotate the hue.
     */
    export function hueRotate(color: Color, angle: number): Color;
    /**
     * Set the alpha value, also known as opacity, of the color.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * const transparent = Color.alpha(blue, 0.1)
     * ```
     * @param color - The original color to modify.
     * @param alpha - A number from 1 to 0. Set to 1 by default.
     */
    export function alpha(color: Color, a?: number): Color;
    /**
     * Set the alpha value, also known as opacity, of the color to zero.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * const transparent = Color.alpha(blue)
     * ```
     * @param color - The original color to modify.
     */
    export function transparent(color: Color): Color;
    /**
     * Change the alpha value, also know as opacity, by a multiplier.
     *
     * @remarks
     * ```typescript
     * const blue = Color("#28AFFA")
     * const transparent = Color.multiplyAlpha(blue, 0.5)
     * ```
     * @param color - The original color to modify.
     * @param a - A number between 1 and 0, defaults to 1,
     */
    export function multiplyAlpha(color: Color, a?: number): Color;
    /**
     * Returns a function that can be used to transition a color from one value
     * to another.
     *
     * @remarks
     * By default this will use the RGB mix model. Useful for providing to animation
     * tools. See also {@link Color.mix()}.
     *
     * ```typescript
     * const state = Color.interpolate(Color("red"), Color("blue"))
     * state(0)   // Initial state (red)
     * state(0.5) // Mid state (purple)
     * state(1)   // Final state (blue)
     * ```
     * @param colorA - The starting color
     * @param colorB - The final color
     * @param model  - The model to use for the mix. One of {@link ColorMixModelType}
     */
    export function interpolate(colorA: Color, colorB: Color, model?: ColorMixModelType): (progress: number) => Color;
    /**
     * Blend two colors together, optionally based on user input. The fraction defines the
     * distribution between the two colors, and is set to 0.5 by default.
     *
     * @remarks
     * ```typescript
     * // Mix red with yellow
     * const orange = Color.mix("red", "yellow", 0.5)
     * ```
     *
     * The `limit` defines if the color can transition beyond its range. This is
     * applicable when transitioning between colors, using Utils.modulate.
     * Below, the limit is set to `true`, so the transition cannot extend beyond
     * the second color.
     *
     * ```typescript
     * fraction = Utils.modulate(offset.x, [0, window.outerWidth], [0, 1], true)
     * Color.mix("red", "yellow", fraction, true, "husl")
     * ```
     *
     * @param colorA   - A color, the first one.
     * @param colorB   - A color, the second one.
     * @param fraction - An optional number, from 0 to 1, set to 0.5 by default.
     * @param limit    - An optional boolean, set to false by default.
     * @param model    - The model to use for the mix. One of {@link ColorMixModelType}
     */
    export function mix(colorA: Color, colorB: Color, fraction?: number, limit?: boolean, model?: ColorMixModelType): Color | null;
    /**
     * Returns a Color instance with a random color value set.
     *
     * @remarks
     * ```typescript
     * const random = Color.random()
     * ```
     *
     * @param a - An optional alpha value, set to 1 by default.
     */
    export function random(a?: number): Color;
    /**
     * Creates a greyscale color.
     *
     * @remarks
     * ```typescript
     * const gray = Color.gray(0.5)
     * ```
     *
     * @param amount - A number from 0 to 1 representing the amount of white.
     * @param alpha  - A number from 0 to 1 representing the alpha. Set to 1 by default.
     */
    export function grey(amount?: number, a?: number): Color;
    /**
     * Alias for {@link Color.gray()}
     */
    const gray: typeof grey;
    /** @internal */
    export function rgbToHsl(r: number, g: number, b: number): ColorHSL;
    /** @internal */
    const isValidColorProperty: (name: string, value: string) => boolean;
    /**
     * Calculates the color difference using {@link https://en.wikipedia.org/wiki/Color_difference#Euclidean |
     * Euclidean distance fitting human perception}. Returns a value between 0 and 765
     * @param colorA - A first color.
     * @param colorB - A second color.
     */
    export function difference(colorA: Color, colorB: Color): number;
    /**
     * Checks whether two Color objects are equal.
     *
     * @remarks
     * ```typescript
     * Color.equal(Color("red"), Color("red"))  //=> true
     * Color.equal(Color("red"), Color("blue")) //=> false
     *
     * Color.equal(Color("#28AFFA"), Color("28AFFB"))    //=> false
     * Color.equal(Color("#28AFFA"), Color("28AFFB"), 2) //=> true
     * ```
     *
     * @param colorA    - The first color
     * @param colorB    - The second color
     * @param tolerance - A tolerance for the difference between rgba values. Set to 0.1 by default.
     */
    export function equal(colorA: Color, colorB: Color, tolerance?: number): boolean;
}

declare interface ColorControlDescription<P = any> extends BaseControlDescription<P> {
    type: ControlType.Color;
    defaultValue?: string;
}

declare enum ColorFormat {
    RGB = "rgb",
    HSL = "hsl",
    HSV = "hsv",
    HEX = "hex",
    NAME = "name"
}

declare interface ColorHSL {
    h: number;
    s: number;
    l: number;
}

declare type ColorHSLA = ColorHSL & {
    a: number;
};

declare interface ColorHSV {
    h: number;
    s: number;
    v: number;
}

declare type ColorHSVA = ColorHSV & {
    a: number;
};

declare enum ColorMixModelType {
    RGB = "rgb",
    RGBA = "rgba",
    HSL = "hsl",
    HSLA = "hsla",
    HUSL = "husl"
}

declare interface ColorRGB {
    r: number;
    g: number;
    b: number;
}

declare type ColorRGBA = ColorRGB & {
    a: number;
};

declare type CompactControlsDescription<P = any> = NumberControlDescription<P> | EnumControlDescription<P>;

/**
 * @internal
 */
export declare class ComponentContainer extends Layer<ComponentContainerProperties, ComponentContainerState> {
    static supportsConstraints: boolean;
    state: ComponentContainerState;
    static defaultComponentContainerProps: ComponentContainerProps;
    static readonly defaultProps: ComponentContainerProperties;
    static rect(props: Partial<ConstraintProperties>): Rect;
    readonly rect: Rect;
    componentDidCatch(error: Error, info: React.ErrorInfo): void;
    constraintValues(): UserConstraintValues;
    render(): JSX.Element;
}

/**
 * @internal
 */
declare interface ComponentContainerProperties extends ComponentContainerProps, LayerProps {
}

/**
 * @internal
 */
declare interface ComponentContainerProps extends ConstraintProperties {
    rotation: Animatable<number> | number;
    opacity: number;
    visible: boolean;
    componentIdentifier: string;
}

/**
 * @internal
 */
declare interface ComponentContainerState {
    lastError?: {
        children: React.ReactNode;
        name: string;
        message: string;
        componentStack: string[];
    };
}

/**
 * @internal
 */
declare type ComponentDefinition<P = any> = {
    class: React.ComponentType<P> | JSON | Override<any>;
    /** Package depth of this component. 0 if part of project, 1 if a direct dependency, greater otherwise. */
    depth: number;
    file: string;
    identifier: ComponentIdentifier;
    name: string;
    /** Identifier of the package that contains this component (one package can contain multiple components). */
    packageIdentifier: PackageIdentifier;
    properties: PropertyControls<P>;
    type: ComponentType;
    defaults?: P;
};

/**
 * @internal
 */
export declare type ComponentIdentifier = string;

declare interface ComponentInstanceDescription<P = any> extends BaseControlDescription<P> {
    type: ControlType.ComponentInstance;
}

/**
 * @ internal
 */
declare interface ComponentLoader {
    /**
     * @internal
     */
    packageDisplayName(packageIdentifier: PackageIdentifier): string | undefined;
    /**
     * @internal
     */
    localPackageIdentifier(): PackageIdentifier;
    /**
     * Names of the packages that are direct dependencies of the project.
     * @internal
     */
    packageIdentifiers(): PackageIdentifier[];
    /**
     * @internal
     */
    componentsForPackage(packageIdentifier: PackageIdentifier): ComponentDefinition[];
    /**
     * @internal
     */
    componentForIdentifier(identifier: ComponentIdentifier): ComponentDefinition | null;
    /**
     * @internal
     */
    errorForIdentifier(identifier: ComponentIdentifier): ErrorDefinition | null;
    /**
     * Identifiers of the components that are in the current project
     * or in packages that are direct dependencies of the project.
     * @internal
     */
    componentIdentifiers(): ComponentIdentifier[];
    /**
     * @internal
     */
    forEachDesignComponents(cb: (component: DesignComponentDefinition) => void): void;
    /**
     * @internal
     */
    forEachComponent(cb: (component: ComponentDefinition) => boolean): void;
}

/**
 * @internal
 */
export declare const componentLoader: ComponentLoader;

/**
 * NOTE: Also defined as ComponentType in the Server project.
 * @internal
 */
declare type ComponentType = "component" | "device" | "deviceHand" | "deviceSkin" | "master" | "override";

declare type ConstraintAuto = "auto";

declare type ConstraintDimension = Animatable<number> | number | ConstraintPercentage | ConstraintAuto | ConstraintFreespaceFraction;

declare type ConstraintFreespaceFraction = string;

declare type ConstraintPercentage = string;

declare interface ConstraintProperties extends Partial<WithFractionOfFreeSpace> {
    parentSize: Size | AnimatableObject<Size> | null;
    left: Animatable<number> | number | null;
    right: Animatable<number> | number | null;
    top: Animatable<number> | number | null;
    bottom: Animatable<number> | number | null;
    centerX: ConstraintPercentage;
    centerY: ConstraintPercentage;
    width: ConstraintDimension;
    height: ConstraintDimension;
    aspectRatio: number | null;
    autoSize?: boolean;
}

declare interface ConstraintValuesBase {
    left: number | null;
    right: number | null;
    top: number | null;
    bottom: number | null;
    centerAnchorX: number;
    centerAnchorY: number;
    widthType: DimensionType;
    heightType: DimensionType;
    aspectRatio: number | null;
}

/** @public */
export declare type ControlDescription<P = any, Q = any> = FlatControlDescription<P> | ArrayControlDescription<P, Q>;

declare type ControlPoints = [number, number, number, number];

/**
 * Used by the {@link PropertyControls} and specifies the type of user
 * interface for receiving input.
 *
 * @remarks
 * Each field has a distinct set of properties though they all accept `title`
 * and `hidden` properties.
 *
 * ```typescript
 * static propertyControls: PropertyControls<Props> = {
 *   text: {
 *     type: ControlType.String,
 *     title: "Text",
 *     hidden: (props) => true  // Return true to hide, receives component props
 *   },
 * }
 * ```
 * @public
 */
export declare const enum ControlType {
    /**
     * A control that displays an on/off checkbox, the associated prop will be true/false
     * depending on the state of the checkbox.
     * @remarks
     * ```typescript
     * static propertyControls: PropertyControls<Props> = {
     *   isCentered: {
     *     type: ControlType.Boolean,
     *     title: "Center",
     *     defaultValue: true, // Optional default state
     *     disabledTitle: "Not Centered", // Optional title when disabled
     *     enabledTitle?: "Centered", // Optional title when enabled
     *   },
     * }
     * ```
     */
    Boolean = "boolean",
    /**
     * A control that accepts a number. This will be provided directly as a prop.
     * @remarks
     * Will display an input field with a range slider by default. The
     * `displayStepper` option can be provided to include a stepper.
     * ```typescript
     * static propertyControls: PropertyControls<Props> = {
     *   offset: {
     *     type: ControlType.Number,
     *     defaultValue: 10, // Optional default value, blank otherwise
     *     max: 1, // Optional upper bound for the input
     *     min: 0, // Optional lower bound for the input
     *     unit: "px", // Optional suffix e.g. "px" to be dispayed next to the value
     *     step: 0.1, // Optional step for the stepper to increment.
     *     displayStepper: true, // If true stepper will be added to input.
     *   },
     * }
     * ```
     */
    Number = "number",
    /**
     * A control that accepts plain text. This will be provided directly as a prop.
     * @remarks
     * Will display an input field with optional placeholder.
     * ```typescript
     * static propertyControls: PropertyControls<Props> = {
     *   offset: {
     *     type: ControlType.String,
     *     defaultValue: "Hello World", // Optional default value, blank otherwise
     *     placeholder: "Enter some text…", // Optional placeholder text for empty input
     *   },
     * }
     * ```
     */
    String = "string",
    /**
     * A control that can be used to take a single number or four distinct fields. These
     * will be mapped across six associated props.
     * @remarks
     * The canonical example for this control is for border, padding or margin control.
     * It will display an input field to accept a single value and a segmented control
     * allowing values to be provided for four distinct fields. A boolean prop can be
     * used in code to determine whether the single or multi input field was used.
     *
     * ```typescript
     * interface Props {
     *   isPerSide: boolean
     *   border: number
     *   top: number
     *   left: number
     *   right: number
     *   bottom: number
     * }
     * static propertyControls: PropertyControls<Props> = {
     *   title: {
     *     type: ControlType.FusedNumber,
     *     // Optional default value for each field.
     *     defaultValue: 0,
     *     // The prop that determines whether the single or multi value input was used.
     *     toggleKey: "isPerSide",
     *     // The individual props to map the values to.
     *     valueKeys: ["top", "left", "right", "bottom"],
     *     // Labels for each of the fields.
     *     valueLabels: ["T", "L", "R", "B"],
     *     // Optional minimum value for each field.
     *     min: 0,
     *   },
     * }
     * ```
     */
    FusedNumber = "fusednumber",
    /**
     * Represents a list of options, the selected option will be provided as a prop.
     * @remarks
     * This control is displayed as a pop-up menu with one of the items in the list
     * selectable.
     *
     * ```typescript
     * interface Props {
     *   value: "a" | "b" | "c"
     * }
     * static propertyControls: PropertyControls<Props> = {
     *   value: {
     *     type: ControlType.Enum,
     *     defaultValue: "a", // Optional selected value
     *     options: ["a", "b", "c"], // The available options
     *     optionTitles: ["Option A", "Option B", "Option C"], Optional human readable titles
     *   },
     * }
     * ```
     */
    Enum = "enum",
    /**
     * Represents a list of options, the selected option will be provided as a prop.
     * @remarks
     * This control is displayed as a segmented control, otherwise exactly the
     * same as {@link ControlType.Enum}.
     *
     * ```typescript
     * interface Props {
     *   value: "a" | "b" | "c"
     * }
     * static propertyControls: PropertyControls<Props> = {
     *   value: {
     *     type: ControlType.SegmentedEnum,
     *     defaultValue: "a", // Optional selected value
     *     options: ["a", "b", "c"], // The available options
     *     optionTitles: ["Option A", "Option B", "Option C"], Optional human readable titles
     *   },
     * }
     * ```
     */
    SegmentedEnum = "segmentedenum",
    /**
     * Represents a color value, will be provided as a string via props.
     * @remarks
     * This control is displayed as a color picker and will provide the selected color in either
     * hex (e.g. `"#afafaf"`) or hsl (e.g. `hsla(203, 87%, 50%, 0.43)`) notation depending on
     * whether there is a alpha channel.
     *
     * ```typescript
     * interface Props {
     *   background: string
     * }
     * static propertyControls: PropertyControls<Props> = {
     *   background: {
     *     type: ControlType.Color,
     *     defaultValue: "#ffffff", // Optional default color
     *   },
     * }
     * ```
     */
    Color = "color",
    /**
     * Represents an image resource, will be provided as url via props.
     * @remarks
     * Displayed as an image well with associated file picker. The chosen asset
     * will be provided as a fully qualified url.
     *
     * ```typescript
     * interface Props {
     *   avatar: string
     * }
     * static propertyControls: PropertyControls<Props> = {
     *   avatar: {
     *     type: ControlType.Image,
     *   },
     * }
     * ```
     */
    Image = "image",
    /**
     * Represents an image resource, will be provided as url via props.
     * @remarks
     * Displayed as an file picker that will open a native file browser. The
     * selected file will be provided as a fully qualified url. The
     * `allowedFileTypes` property must be provided to specify acceptable file
     * types.
     *
     * ```typescript
     * interface Props {
     *   asset: string
     * }
     * static propertyControls: PropertyControls<Props> = {
     *   asset: {
     *     type: ControlType.File,
     *     allowedFileTypes: ["pdf"], // Accepted file types
     *   },
     * }
     * ```
     */
    File = "file",
    /**
     * A reference to another component on the canvas, provided via a prop.
     * @remarks
     * The component will have an outlet to allow dragging a link to another
     * Frame. Available frames will also be displayed in a pop-up in the
     * properties panel. The component reference will be provided as a prop,
     * as a convention the name for the prop is usually just `children`.
     *
     * Multiple components can be linked by combining the `ComponentInstance`
     * type with the {@link ControlType.Array}.
     *
     * ```typescript
     * interface Props {
     *   children: React.ReactNode
     * }
     * static propertyControls: PropertyControls<Props> = {
     *   children: {
     *     type: ControlType.ComponentInstance,
     *   },
     * }
     * ```
     */
    ComponentInstance = "componentinstance",
    /**
     * Allows multiple values per ControlType, provided as an array via props.
     * @remarks
     * For most control types this control will be displayed as an additional
     * section within the properties panel allowing as many fields to be provided
     * as required.
     *
     * For a {@link ControlType.ComponentInstance} the Frame will also gain
     * an additional outlet UI that allows links to be dragged between frames.
     *
     * ```typescript
     * interface Props {
     *   images: string[],
     *   children: React.ReactNode[]
     * }
     * static propertyControls: PropertyControls<Props> = {
     *   images: {
     *     type: ControlType.Array,
     *     propertyControl: {
     *       type: ControlType.Image
     *     }
     *   },
     *   children: {
     *     type: ControlType.Array,
     *     propertyControl: {
     *       type: ControlType.ComponentInstance
     *     }
     *   },
     * }
     * ```
     */
    Array = "array"
}

/**
 * @beta
 */
export declare namespace ConvertColor {
    export function hueRotate(color: string, angle: number): string;
    export function setAlpha(color: string, alpha: number): string;
    export function getAlpha(color: string): number;
    export function multiplyAlpha(color: string, alpha: number): string;
    export function toHex(color: string): string;
    export function toRgb(color: string): ColorRGBA;
    export function toRgbString(color: string): string;
    export function toHSV(color: string): ColorHSVA;
    export function toHSL(color: string): ColorHSLA;
    export function toHslString(color: string): string;
    export function toHsvString(color: string): string;
    export function hsvToHSLString(hsv: ColorHSV | ColorHSVA): string;
    export function hsvToString(hsv: ColorHSV | ColorHSVA): string;
    export function rgbaToString(color: ColorRGB | ColorRGBA): string;
    export function toColorPickerSquare(h: number): string;
    export function isValid(color: string): boolean;
    export function equals(a: Color | string, b: Color | string): boolean;
    export function toHexOrRgbaString(input: string): string;
}

declare interface CoreFrameProps extends FrameProperties, LayerProps {
}

/**
 * @internal
 */
export declare function createDesignComponent<P>(canvas: CanvasStore, id: string, propertyControls: PropertyControls<P>, width?: ConstraintDimension, height?: ConstraintDimension): {
    new (props: P & Partial<ConstraintProperties>, context?: any): {
        checkedParent: boolean;
        parentError: boolean;
        hasParentError(): boolean;
        componentWillUnmount(): void;
        _typeForName(name: any): any;
        _createElement(type: any, props: any, children: any[]): React.ComponentElement<any, React.Component<any, any, any>>;
        _renderData(presentation: any, componentProps: any, topLevelProps?: any): React.ComponentElement<any, React.Component<any, any, any>> | null;
        render(): React.ComponentElement<any, React.Component<any, any, any>> | React.DetailedReactHTMLElement<{
            style: React.CSSProperties;
        }, HTMLElement> | null;
        setState<K extends "data">(state: {
            data: any;
        } | ((prevState: Readonly<{
            data: any;
        }>, props: Readonly<P & Partial<ConstraintProperties>>) => {
            data: any;
        } | Pick<{
            data: any;
        }, K> | null) | Pick<{
            data: any;
        }, K> | null, callback?: (() => void) | undefined): void;
        forceUpdate(callBack?: (() => void) | undefined): void;
        readonly props: Readonly<{
            children?: React.ReactNode;
        }> & Readonly<P & Partial<ConstraintProperties>>;
        state: Readonly<{
            data: any;
        }>;
        context: any;
        refs: {
            [key: string]: React.ReactInstance;
        };
    };
    propertyControls: PropertyControls<P, any>;
    supportsConstraints: boolean;
    defaultProps: ConstraintProperties;
    rect(props: Partial<ConstraintProperties>): Rect;
    minSize(props: Partial<ConstraintProperties>, parentSize: any): Size;
    size(props: Partial<ConstraintProperties>, parentSize: any, freeSpace: WithFractionOfFreeSpace): Size;
};

declare type Curve = ControlPoints | Bezier;

declare type CustomPageEffect = (info: PageEffectInfo) => Partial<FrameProperties>;

declare interface DampingDurationSpringOptions {
    dampingRatio: number;
    duration: number;
    velocity: number;
    mass: number;
}

/**
 * @public
 */
export declare function Data<T extends object = object>(initial?: Partial<T> | object): T;

/**
 * @public
 */
export declare namespace Data {
    /**
     * @internal
     */
    let _stores: object[];
    /** @internal */
    export function addData(data: object): void;
    /** @internal */
    export function addObserver<T extends object>(target: T, observer: Observer<T>): Cancel;
}

/**
 * @internal
 */
export declare class DataObserver extends React.Component<Props, State_2> {
    observers: Cancel[];
    state: {
        update: number;
    };
    static getDerivedStateFromProps(nextProps: Props, prevState: State_2): State_2 | null;
    taskAdded: boolean;
    frameTask: () => void;
    observer: () => void;
    render(): JSX.Element;
}

/**
 * @internal
 */
declare type DesignComponentDefinition = ComponentDefinition & {
    class: JSONObject;
};

/**
 * @internal
 */
export declare class Device extends React.Component<Partial<DeviceProperties>> {
    /**
     * @internal
     */
    static defaultProps: DeviceProperties;
    /**
     * @internal
     */
    static registry: DeviceRegistry;
    /**
     * @internal
     */
    static descriptor: DeviceDescriptor;
    /**
     * @internal
     */
    readonly descriptor: DeviceDescriptor;
    /**
     * @internal
     */
    readonly skins: DeviceSkins;
    /**
     * @internal
     */
    readonly hands: DeviceHands;
    /**
     * @internal
     */
    readonly svgScreenMask: string | undefined;
    props: Readonly<DeviceProperties> & Readonly<{
        children?: ReactNode;
    }>;
    /**
     * @internal
     */
    componentDidMount(): void;
    /**
     * @internal
     */
    componentDidUpdate(prevProps: DeviceProperties): void;
    /**
     * @internal
     */
    computeRequiredPackages(): void;
    /**
     * @internal
     */
    render(): JSX.Element;
    private getSkin;
    private getHand;
    private outerStyle;
    private static skinOrHandFromPackage;
    private toLocalPath;
}

/**
 * @internal
 */
export declare interface DeviceDescriptor {
    title: string;
    category?: string;
    selector?: string;
    kind?: string;
    skins: DeviceSkins;
    hands: DeviceHands;
    screen: Size;
    pixelRatio?: number;
    canRotate?: boolean;
}

/**
 * @alpha
 */
declare type DeviceHand = {
    image: string;
    width: number;
    height: number;
    offset: number;
};

/**
 * @alpha
 */
declare type DeviceHands = {
    [key: string]: DeviceHand | ExternalDeviceHand;
};

/**
 * @internal
 */
export declare interface DeviceProperties {
    parentSize: null | AnimatableObject<Size>;
    deviceSize: Size;
    contentSize: Size;
    skin?: string;
    hand?: string;
    zoom: number;
    responsive: boolean;
    rotate?: boolean;
    background?: string;
    onRequirePackage: (pakkage: string, displayName: string) => void;
    /**
     * @internal
     */
    renderer: typeof DeviceRenderer;
}

/**
 * @internal
 */
declare class DeviceRegistry {
    resolve(identifier: string): typeof Device | undefined;
    list(): DevicesData;
}

/** @internal */
export declare class DeviceRenderer extends React.Component<DeviceRendererProperties> {
    static defaultProps: DeviceRendererProperties;
    static getMode(props: DeviceRendererProperties): DeviceRendererMode;
    static getBackgroundColor(props: DeviceRendererProperties): string;
    private isViewingOnMobile;
    getScreenStyle(screen: Size, device: Size, rotate: boolean, scale: number, svgScreenMask?: string): React.CSSProperties;
    private willRenderWithScale;
    render(): JSX.Element | JSX.Element[];
    renderCanvasMode(): JSX.Element;
    renderScreenOnly(): JSX.Element;
    renderSkinAndScreen(skin: DeviceSkin, hand: DeviceHand | null): JSX.Element[];
    calculateSkinRectAndScreenScale(skin: DeviceSkin, outerSize: Size, rotate: boolean): {
        width: number;
        height: number;
        left: number;
        top: number;
        scale: number;
    };
    calculateScreenRect(screen: Size, outerSize: Size, rotate: boolean, scale: number): {
        width: number;
        height: number;
        left: number;
        top: number;
    };
}

/** @internal */
declare enum DeviceRendererMode {
    Canvas = 0,
    Screen = 1,
    Device = 2
}

/** @internal */
declare interface DeviceRendererProperties {
    skin: DeviceSkin | null;
    hand: DeviceHand | null;
    device: Size;
    screen: Size;
    content: Size;
    pixelRatio: number;
    rotate: boolean;
    responsive: boolean;
    svgScreenMask?: string;
}

/**
 * @internal
 */
declare type DevicesData = {
    devices: {
        [key: string]: ComponentDefinition;
    };
    deviceSkins: {
        [key: string]: ComponentDefinition;
    };
};

/**
 * @internal
 */
export declare interface DeviceSkin {
    image: string;
    imageWidth: number;
    imageHeight: number;
    padding: number;
    background: string;
}

/**
 * @internal
 */
export declare type DeviceSkins = {
    [key: string]: DeviceSkin | ExternalDeviceSkin;
};

declare enum DimensionType {
    FixedNumber = 0,
    Percentage = 1,
    /** @internal */ Auto = 2,
    FractionOfFreeSpace = 3
}

declare type DragEventHandler<Draggable> = (event: FramerEvent, draggable: Draggable) => void;

declare interface DragEvents<Draggable> {
    onMove: (point: Point, draggable: Draggable) => void;
    /**
     * @beta
     */
    onDragDirectionLockStart: (axis: Axis, draggable: Draggable) => void;
    onDragAnimationStart: (animation: {
        x: AnimationInterface;
        y: AnimationInterface;
    }, draggable: Draggable) => void;
    onDragAnimationEnd: (animation: {
        x: AnimationInterface;
        y: AnimationInterface;
    }, draggable: Draggable) => void;
    onDragSessionStart: DragEventHandler<Draggable>;
    onDragSessionMove: DragEventHandler<Draggable>;
    onDragSessionEnd: DragEventHandler<Draggable>;
    onDragStart: DragEventHandler<Draggable>;
    onDragWillMove: DragEventHandler<Draggable>;
    onDragDidMove: DragEventHandler<Draggable>;
    onDragEnd: DragEventHandler<Draggable>;
}

/** @public */
export declare const Draggable: React.ComponentClass<Partial<FrameProps> & Partial<DraggableProps<typeof Frame>>>;

declare interface DraggableProps<Draggable> extends DraggableSpecificProps<Draggable> {
    enabled: boolean;
}

declare interface DraggableSpecificProps<Draggable> extends Partial<DragEvents<Draggable>> {
    momentum: boolean;
    momentumOptions: {
        friction: number;
        tolerance: number;
    };
    momentumVelocityMultiplier: number;
    speedX: number;
    speedY: number;
    bounce: boolean;
    bounceOptions: {
        friction: number;
        tension: number;
        tolerance: number;
    };
    directionLock: boolean;
    directionLockThreshold: {
        x: number;
        y: number;
    };
    overdrag: boolean;
    overdragScale: number;
    pixelAlign: boolean;
    velocityTimeout: number;
    velocityScale: number;
    horizontal: boolean;
    vertical: boolean;
    constraints: Partial<Rect>;
    mouseWheel: boolean;
}

declare type DraggableType = typeof Frame;

/**
 * @internal
 */
declare interface DriverClass<AnimatorClass extends Animator<Value, Options>, Value, Options = any> {
    new (animator: AnimatorClass, updateCallback: (value: Value) => void, doneCallback?: (isFinished: boolean) => void): AnimationDriver<AnimatorClass, Value, Options>;
}

declare type EaseOptions = Omit<BezierOptions, "curve">;

declare interface EnumControlDescription<P = any> extends BaseControlDescription<P> {
    type: ControlType.Enum;
    defaultValue?: string;
    options: string[];
    optionTitles?: string[] | ((props: P | null) => string[]);
}

/**
 * @internal
 */
declare type ErrorDefinition = ComponentDefinition<{}> & {
    error: Error | string;
    fileDoesNotExist?: boolean;
};

declare type EventDispatcher = ((type: string, event: FramerEvent, target: EventTarget) => void);

declare class EventEmitter<EventName> {
    private _emitter;
    eventNames(): string[];
    eventListeners(): {
        [index: string]: ListenerFn[];
    };
    on(eventName: EventName, fn: Function): void;
    off(eventName: EventName, fn: Function): void;
    once(eventName: EventName, fn: Function): void;
    unique(eventName: EventName, fn: Function): void;
    addEventListener(eventName: EventName, fn: Function, once: boolean, unique: boolean, context: Object): void;
    removeEventListeners(eventName?: EventName, fn?: Function): void;
    removeAllEventListeners(): void;
    countEventListeners(eventName?: EventName, handler?: Function): number;
    emit(eventName: EventName, ...args: any[]): void;
}

declare type EventHandler = (event: FramerEvent) => void;

/**
 * @alpha
 */
declare type ExternalDeviceHand = WithPackage & Partial<DeviceHand>;

/**
 * @internal
 */
declare type ExternalDeviceSkin = WithPackage & Partial<DeviceSkin>;

declare interface FileControlDescription<P = any> extends BaseControlDescription<P> {
    type: ControlType.File;
    allowedFileTypes: string[];
}

declare interface FillProperties {
    fill: Animatable<Background> | Background | null;
}

declare interface FilterNumberProperties {
    brightness: number;
    contrast: number;
    grayscale: number;
    hueRotate: number;
    invert: number;
    saturate: number;
    sepia: number;
    blur: number;
}

declare interface FilterProperties extends FilterNumberProperties {
    dropShadows: Shadow[];
}

/**
 * @public
 */
declare type FinishFunction = (transaction: TransactionId) => void;

declare type FlatControlDescription<P = any> = CompactControlsDescription<P> | BooleanControlDescription<P> | StringControlDescription<P> | ColorControlDescription<P> | FusedNumberControlDescription<P> | SegmentedEnumControlDescription<P> | ImageControlDescription<P> | FileControlDescription<P> | ComponentInstanceDescription<P>;

/** @public */
export declare const Frame: React.ComponentClass<Partial<FrameProps>>;

/** @public */
export declare interface FrameProperties extends ConstraintProperties, TransformProperties, VisualProperties {
    visible: boolean;
    name?: string;
    backfaceVisible?: boolean | Animatable<boolean>;
    perspective?: number | Animatable<number>;
    preserve3d?: boolean | Animatable<boolean>;
    borderWidth: number | Partial<{
        top: number;
        bottom: number;
        left: number;
        right: number;
    }>;
    borderColor: string;
    borderStyle: BorderStyle;
    style?: React.CSSProperties;
    className?: string;
    /** @internal */
    _overrideForwardingDescription?: {
        [key: string]: string;
    };
}

/** @public */
export declare type FrameProps = CoreFrameProps & WithEventsProperties;

/**
 * @privateRemarks
 * This could be called just Animation, but it's type would clash with
 * javascript's native Animation: https://developer.mozilla.org/en-US/docs/Web/API/Animation
 * So if you forget the import, you would get weird errors.
 *
 * Also, this class follows the native Animation as much as possible.
 * @public
 */
export declare class FramerAnimation<Value, AnimatorOptions> {
    /**
     * @internal
     */
    private driver;
    /**
     * @internal
     */
    constructor(target: Animatable<Value> | AnimatableObject<Value>, from: Value, to: Value, animatorClass?: AnimatorClass<Value, AnimatorOptions>, options?: Partial<AnimatorOptions & AnimationOptions<Value>>, driverClass?: DriverClass<Animator<Value, AnimatorOptions>, Value, AnimatorOptions>);
    /**
     * @internal
     */
    private static driverCallbackHandler;
    /**
     * @internal
     */
    private playStateSource;
    /**
     * @internal
     */
    /**
    * @internal
    */
    private playStateValue;
    /**
     * @alpha
     */
    readonly playState: FramerAnimationState;
    /**
     * @alpha
     */
    onfinish: undefined | (() => void);
    /**
     * @alpha
     */
    oncancel: undefined | (() => void);
    /**
     * @internal
     */
    private readyPromise;
    /**
     * @internal
     */
    private readyResolve;
    /**
     * @internal
     */
    private resetReadyPromise;
    /**
     * @public
     */
    readonly ready: Promise<void>;
    /**
     * @internal
     */
    private finishedPromise;
    /**
     * @internal
     */
    private finishedResolve;
    /**
     * @internal
     */
    private finishedReject;
    /**
     * @internal
     */
    private resetFinishedPromise;
    /**
     * @public
     */
    readonly finished: Promise<void>;
    /**
     * @alpha
     */
    play(): void;
    /**
     * @public
     */
    cancel(): void;
    /**
     * @alpha
     */
    finish(): void;
    /**
     * @alpha
     */
    isFinished(): boolean;
}

declare type FramerAnimationState = "idle" | "running" | "finished";

/** @internal */
export declare const FramerAppleIMac: typeof Device;

/** @internal */
export declare const FramerAppleIPadAir: typeof Device;

/** @internal */
export declare const FramerAppleIPadMini: typeof Device;

/** @internal */
export declare const FramerAppleIPadPro: typeof Device;

/** @internal */
export declare const FramerAppleIPhone8: typeof Device;

/** @internal */
export declare const FramerAppleIPhone8Plus: typeof Device;

/** @internal */
export declare const FramerAppleIPhoneSE: typeof Device;

/** @internal */
export declare const FramerAppleIPhoneX: typeof Device;

/** @internal */
export declare const FramerAppleIPhoneXR: typeof Device;

/** @internal */
export declare const FramerAppleIPhoneXS: typeof Device;

/** @internal */
export declare const FramerAppleIPhoneXSMax: typeof Device;

/** @internal */
export declare const FramerAppleMacBook: typeof Device;

/** @internal */
export declare const FramerAppleMacBookAir: typeof Device;

/** @internal */
export declare const FramerAppleMacBookPro: typeof Device;

/** @internal */
export declare const FramerAppleThunderboltDisplay: typeof Device;

/** @internal */
export declare const FramerAppleWatch38: typeof Device;

/** @internal */
export declare const FramerAppleWatch42: typeof Device;

/** @internal */
export declare const FramerDellXPS: typeof Device;

/**
 * @public
 */
export declare class FramerEvent {
    /** @internal */ readonly originalEvent: MouseEvent | TouchEvent;
    /** @internal */ readonly session?: FramerEventSession | undefined;
    /**
     * @internal
     */
    readonly time: number;
    /**
     * @internal
     */
    readonly loopTime: number;
    /**
     * @internal
     */
    readonly point: Point;
    /**
     * @internal
     */
    readonly devicePoint: Point;
    /**
     * @internal
     */
    readonly target: EventTarget | null;
    /**
     * @internal
     */
    readonly delta: Point;
    /**
     * @internal
     */
    constructor(
    /** @internal */ originalEvent: MouseEvent | TouchEvent, 
    /** @internal */ session?: FramerEventSession | undefined);
    private static eventLikeFromOriginalEvent;
    /**
     * @internal
     */
    velocity(t: number): Point;
    /**
     * @internal
     */
    readonly offset: Point;
    /**
     * @internal
     */
    readonly isLeftMouseClick: boolean | undefined;
}

/** @internal */
export declare const FramerEventListener: React.ComponentType<any>;

/**
 * @alpha
 */
export declare class FramerEventSession implements GestureHandler {
    private events;
    private recognizers;
    private mouseWheelRecognizer;
    private dispatcher;
    /**
     * @internal
     */
    originElement: HTMLElement;
    readonly isStarted: boolean;
    readonly startEvent: FramerEvent | null;
    readonly lastEvent: FramerEvent | null;
    constructor(dispatcher: EventDispatcher, customOrigin?: HTMLElement);
    private processEvent;
    pointerDown(event: FramerEvent): void;
    pointerMove(event: FramerEvent): void;
    pointerUp(event: FramerEvent): void;
    mouseWheel(event: FramerEvent): void;
    private dispatch;
    gestureBegan(type: string, event: FramerEvent, target: EventTarget | null): void;
    gestureChanged(type: string, event: FramerEvent, target: EventTarget | null): void;
    gestureEnded(type: string, event: FramerEvent, target: EventTarget | null): void;
    /**
     * Average velocity over last n seconds in pixels per second.
     * @param n - number of events to use for calculation
     */
    velocity(t?: number): Point;
    offset(event: FramerEvent): Point;
}

/** @internal */
export declare const FramerGoogleNexus4: typeof Device;

/** @internal */
export declare const FramerGoogleNexus5X: typeof Device;

/** @internal */
export declare const FramerGoogleNexus6: typeof Device;

/** @internal */
export declare const FramerGoogleNexusTablet: typeof Device;

/** @internal */
export declare const FramerGooglePixel2: typeof Device;

/** @internal */
export declare const FramerGooglePixel2XL: typeof Device;

/** @internal */
export declare const FramerGooglePixel3: typeof Device;

/** @internal */
export declare const FramerGooglePixel3XL: typeof Device;

/** @internal */
export declare const FramerHTCOneA9: typeof Device;

/** @internal */
export declare const FramerMicrosoftLumia950: typeof Device;

/** @internal */
export declare const FramerMicrosoftSurfaceBook: typeof Device;

/** @internal */
export declare const FramerMicrosoftSurfacePro3: typeof Device;

/** @internal */
export declare const FramerMicrosoftSurfacePro4: typeof Device;

/** @internal */
export declare const FramerSamsungGalaxyS8: typeof Device;

/** @internal */
export declare const FramerSamsungGalaxyS9: typeof Device;

/** @internal */
export declare const FramerSamsungNote5: typeof Device;

/** @internal */
export declare const FramerSonySmartWatch: typeof Device;

/** @internal */
export declare const FramerSonyW850C: typeof Device;

/** @internal */
export declare const FramerStoreArtwork: typeof Device;

/** @internal */
export declare const FramerStoreIcon: typeof Device;

declare interface FusedNumberControlDescription<P = any> extends BaseControlDescription<P> {
    type: ControlType.FusedNumber;
    defaultValue?: number;
    toggleKey: keyof P;
    toggleTitles: [string, string];
    valueKeys: [keyof P, keyof P, keyof P, keyof P];
    valueLabels: [string, string, string, string];
    min?: number;
}

/**
 * @internal
 */
declare interface GestureHandler {
    gestureBegan: (type: string, event: FramerEvent, target: EventTarget | null) => void;
    gestureChanged: (type: string, event: FramerEvent, target: EventTarget | null) => void;
    gestureEnded: (type: string, event: FramerEvent, target: EventTarget | null) => void;
}

/**
 * @internal
 */
export declare function getURLs(): {
    imageBaseURL: string;
    projectURL: string;
};

/**
 * @public
 */
declare type Gradient = LinearGradient | RadialGradient;

/** @public */
declare interface IdentityProps extends React.Props<any> {
    id?: string;
}

declare interface ImageControlDescription<P = any> extends BaseControlDescription<P> {
    type: ControlType.Image;
}

declare type ImageFit = "fill" | "fit" | "stretch";

declare type IncomingColor = ColorRGB | ColorHSL | ColorRGBA | ColorHSLA | string;

/**
 * @public
 */
declare interface Interpolation<Value = any> {
    /**
     * @beta
     */
    interpolate(from: Value, to: Value): Interpolator<Value>;
    /**
     * difference(from, to) calculates a measure of difference between two values,
     * such that for every value of from, to and x holds:
     * interpolator = interpolate(from, to)
     * total = difference(from, to)
     * interpolator( difference(from, x) / total ) === x
     * @beta
     */
    difference(from: Value, to: Value): number;
}

/**
 * @public
 */
declare namespace Interpolation {
    /**
     * @param from
     * @param to
     * @beta
     */
    function handleUndefined<Value>(from: Value, to: Value): [Value, Value];
}

declare interface InterpolationOptions {
    colorModel: ColorMixModelType;
}

/**
 * @beta
 */
declare type Interpolator<Value> = (progress: number) => Value;

/**
 * @internal
 */
export declare function isOverride(d: ComponentDefinition): boolean;

/**
 * @internal
 */
export declare function isReactDefinition<P = any>(d: ComponentDefinition<P>): d is ReactComponentDefinition<P>;

/**
 * @internal
 */
declare interface JSONArray extends Array<JSONData> {
}

/**
 * @internal
 */
declare type JSONData = null | string | number | boolean | JSONArray | JSONObject;

/**
 * @internal
 */
declare type JSONObject = {
    [key: string]: JSONData;
};

/**
 * @public
 */
declare class Layer<P extends Partial<LayerProps>, S> extends React.Component<P, S> {
    static readonly defaultProps: LayerProps;
    static applyWillChange<P extends Partial<LayerProps>>(layer: React.Component<P>, style: React.CSSProperties): void;
    /** @internal */
    shouldComponentUpdate(nextProps: P, nextState: S): boolean;
    props: Readonly<P> & Readonly<{
        children?: ReactNode;
    }>;
    private previousZoom;
    /** @internal */
    componentDidUpdate(prevProps: P): void;
    /** @internal */
    protected resetSetStyle(key: string, toValue: any | null, microtask?: boolean): void;
}

/** @public */
declare interface LayerProps extends IdentityProps {
    willChangeTransform?: boolean;
    /** @internal */
    _forwardedOverrides?: {
        [key: string]: any;
    };
}

/**
 * @alpha
 */
declare function Line(a: Point, b: Point): Line;

/**
 * @alpha
 */
declare interface Line {
    a: Point;
    b: Point;
}

/**
 * @alpha
 */
declare namespace Line {
    function intersection(lineA: Line, lineB: Line): Point | null;
    const isOrthogonal: (line: Line) => boolean;
    const perpendicular: (line: Line, pointOnLine: Point) => Line;
    function projectPoint(line: Line, point: Point): Point | null;
}

/**
 * @public
 */
declare interface LinearGradient {
    alpha: number;
    angle: number;
    start: string;
    end: string;
}

/**
 * @public
 */
declare namespace LinearGradient {
    /**
     * @param linearGradient
     */
    function isLinearGradient(linearGradient: any): linearGradient is LinearGradient;
    /** @internal */
    function hash(linearGradient: LinearGradient): number;
    /** @alpha */
    function toCSS(linearGradient: LinearGradient, overrideAngle?: number): string;
}

/**
 * @alpha
 */
declare type LineCap = "butt" | "round" | "square";

/**
 * @alpha
 */
declare type LineJoin = "miter" | "round" | "bevel";

/**
 * @alpha
 */
export declare function loadJSON<T>(url: string): Promise<T>;

/**
 * @public
 */
declare class Loop extends EventEmitter<LoopEventNames> {
    private _started;
    private _frame;
    private _frameTasks;
    /**
     * To add a task to be done at the end of a frame.
     * Tasks added from a task will be ignored. These will run after loop events have been processed.
     * @internal
     */
    addFrameTask(task: Function): void;
    private _processFrameTasks;
    /**
     * @internal
     */
    /**
    * @internal
    */
    static TimeStep: number;
    /**
     * @internal
     */
    constructor(start?: boolean);
    /**
     * @internal
     */
    start(): this;
    /**
     * @internal
     */
    stop(): this;
    /**
     * @internal
     */
    readonly frame: number;
    /**
     * @internal
     */
    readonly time: number;
    /**
     * @internal
     */
    private tick;
}

declare type LoopEventNames = "render" | "update" | "finish";

/**
 * @internal
 */
export declare const MainLoop: Loop;

/**
 * @internal
 */
declare enum NavigateTo {
    Previous = "@Previous"
}

/**
 * @internal
 */
export declare class Navigation extends React.Component<NavigationProps, NavigationState> implements Navigator {
    stack: NavigationItem[];
    overlayStack: NavigationItem[];
    activeTransition: NavigationTransitionOptions;
    stackItemID: number;
    state: {
        current: number;
        previous: number;
        currentOverlay: number;
        previousOverlay: number;
    };
    componentDidMount(): void;
    componentWillReceiveProps(props: NavigationProps): void;
    getStackState(overCurrentContext: boolean): StackState_2;
    newStackItem(Component: React.ReactNode, transition: NavigationTransitionOptions): {
        key: string;
        component: React.ReactNode;
        transition: NavigationTransitionOptions;
    };
    push: (Component: React.ReactNode, transition?: NavigationTransitionOptions, transitionOverrides?: NavigationTransitionOptions | undefined) => void;
    goBack: () => void;
    instant: (Component: React.ReactNode) => void;
    fade: (Component: React.ReactNode) => void;
    pushLeft: (Component: React.ReactNode) => void;
    pushRight: (Component: React.ReactNode) => void;
    pushDown: (Component: React.ReactNode) => void;
    pushUp: (Component: React.ReactNode) => void;
    modal: (Component: React.ReactNode, overrides?: NavigationTransitionOptions | undefined) => void;
    overlayTop: (Component: React.ReactNode, overrides?: NavigationTransitionOptions | undefined) => void;
    overlayBottom: (Component: React.ReactNode, overrides?: NavigationTransitionOptions | undefined) => void;
    overlayLeft: (Component: React.ReactNode, overrides?: NavigationTransitionOptions | undefined) => void;
    overlayRight: (Component: React.ReactNode, overrides?: NavigationTransitionOptions | undefined) => void;
    flipUp: (Component: React.ReactNode) => void;
    flipDown: (Component: React.ReactNode) => void;
    flipLeft: (Component: React.ReactNode) => void;
    flipRight: (Component: React.ReactNode) => void;
    contextSize(): Size;
    contentSize(containerIndex: number, contextSize: Size, stackState: StackState_2): Size;
    render(): JSX.Element;
}

/**
 * @internal
 */
declare type NavigationItem = {
    key: string;
    component: React.ReactNode;
    transition: NavigationTransitionOptions;
};

/**
 * @internal
 */
export declare interface NavigationLink {
    navigationTarget: NavigationTarget;
    navigationTransition: NavigationTransition;
    navigationTransitionDirection: NavigationTransitionDirection;
    navigationTransitionOverrides?: NavigationTransitionOptions;
}

/**
 * @internal
 */
declare type NavigationProps = Partial<Rect>;

/**
 * @internal
 */
declare interface NavigationState {
    current: number;
    previous: number;
    currentOverlay: number;
    previousOverlay: number;
}

/**
 * @internal
 */
declare type NavigationTarget = string | NavigateTo.Previous;

/**
 * @internal
 */
declare enum NavigationTransition {
    push = "push",
    instant = "instant",
    fade = "fade",
    modal = "modal",
    overlay = "overlay",
    flip = "flip"
}

/**
 * @internal
 */
declare enum NavigationTransitionDirection {
    left = "left",
    right = "right",
    up = "up",
    down = "down"
}

/**
 * @internal
 */
export declare interface NavigationTransitionOptions {
    fitWidth?: boolean;
    fitHeight?: boolean;
    transitionDuration?: number;
    overCurrentContext?: boolean;
    goBackOnTapOutside?: boolean;
    backdropColor?: string;
    inStart?(contextSize: Size, contentSize: Size): Partial<FrameProperties>;
    inEnd?(contextSize: Size, contentSize: Size): Partial<FrameProperties>;
    outEnd?(contextSize: Size, contentSize: Size): Partial<FrameProperties>;
}

/**
 * @internal
 */
declare interface Navigator {
    goBack: () => void;
    instant: (Component: React.ReactNode) => void;
    fade: (component: React.ReactNode) => void;
    push: (Component: React.ReactNode) => void;
    pushLeft: (Component: React.ReactNode) => void;
    pushRight: (Component: React.ReactNode) => void;
    pushDown: (Component: React.ReactNode) => void;
    pushUp: (Component: React.ReactNode) => void;
    modal: (Component: React.ReactNode, overrides?: NavigationTransitionOptions) => void;
    overlayTop: (Component: React.ReactNode, overrides?: NavigationTransitionOptions) => void;
    overlayBottom: (Component: React.ReactNode, overrides?: NavigationTransitionOptions) => void;
    overlayLeft: (Component: React.ReactNode, overrides?: NavigationTransitionOptions) => void;
    overlayRight: (Component: React.ReactNode, overrides?: NavigationTransitionOptions) => void;
    flipUp: (Component: React.ReactNode) => void;
    flipDown: (Component: React.ReactNode) => void;
    flipLeft: (Component: React.ReactNode) => void;
    flipRight: (Component: React.ReactNode) => void;
}

declare interface NumberControlDescription<P = any> extends BaseControlDescription<P> {
    type: ControlType.Number;
    defaultValue?: number;
    max?: number;
    min?: number;
    unit?: string;
    step?: number;
    displayStepper?: boolean;
}

/**
 * @beta
 */
export declare function ObservableObject<T extends object = object>(initial?: Partial<T> | object, makeAnimatables?: boolean, observeAnimatables?: boolean): AnimatableObject<T>;

/**
 * @beta
 */
export declare namespace ObservableObject {
    export function addObserver<T extends object>(target: T, observer: Observer<T>): Cancel;
}

/**
 * @public
 */
declare type Observer<Value> = {
    update: UpdateFunction<Value>;
    finish: FinishFunction;
} | UpdateFunction<Value>;

declare type Omit<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;

declare type Overflow = "visible" | "hidden" | "scroll" | "auto";

declare interface OverflowProperties {
    overflow: Overflow;
}

/** @public */
export declare type Override<T extends object = FrameProps & {
    [key: string]: any;
}> = OverrideObject<T> | OverrideFunction<T>;

/** @public */
export declare type OverrideFunction<P extends object = any> = (props: P) => Partial<P>;

/** @public */
export declare type OverrideObject<T extends object = any> = Partial<T>;

/**
 * @internal
 */
declare type PackageIdentifier = string;

/**
 * The Page component allows you to create horizontally or vertically swipeable areas. It can be imported from the Framer Library and used in code components. Add children to create pages to swipe between. These children will be stretched to the size of the page component by default, but can also be set to auto to maintain their original size.
 * See {@link PageProperties} for it's properties
 * @public
 */
export declare class Page extends React.Component<Partial<PageProps>> {
    /** @internal */
    static _isStylable: boolean;
    private pages;
    private animation;
    private propsBoundedCurrentPage;
    private shouldReflectPropsBoundedCurrentPage;
    private contentOffset;
    private currentContentPage;
    private cancelUpdating;
    private static pageProps;
    /** @internal */
    static defaultProps: PageProps;
    /** @internal */
    static propertyControls: PropertyControls<PageProps>;
    /** @internal */
    props: PageProps;
    private readonly isHorizontalDirection;
    private readonly currentContentOffset;
    private boundedCurrentPage;
    /**
     * @internal
     */
    componentWillUpdate(nextProps: PageProps): void;
    /**
     * @internal
     */
    componentDidUpdate(): void;
    private nearestPageIndex;
    private onDragAnimationStart;
    private onDragSessionStart;
    private applyEffects;
    private contentOffsetUpdated;
    private cancelAnimation;
    private effectValues;
    private wrapHandler;
    private offsetForPage;
    /**
     * @internal
     */
    render(): JSX.Element;
}

declare enum PageAlignment {
    Start = "start",
    Center = "center",
    End = "end"
}

declare type PageContentDimension = "auto" | "stretch";

declare type PageDirection = "horizontal" | "vertical";

/**
 * @public
 */
export declare enum PageEffect {
    None = "none",
    Cube = "cube",
    CoverFlow = "coverflow",
    Wheel = "wheel",
    Pile = "pile"
}

declare interface PageEffectInfo {
    offset: number;
    normalizedOffset: number;
    size: Size;
    index: number;
    direction: PageDirection;
    gap: number;
    pageCount: number;
}

/**
 * Event callbacks for the Page component, can be used to react to and co-ordinate
 * with other components.
 *
 * @remarks
 * ```typescript
 * import { Page, Frame } from "framer"
 * const MyPage: React.SFC = props => {
 *   return (
 *     <Page {...props} onChangePage={idx => console.log(idx)}>
 *       <Frame>Hello</Frame>
 *       <Frame>World</Frame>
 *     </Page>
 *   );
 * };
 * ```
 * @public
 */
export declare interface PageEvents {
    /**
     * A callback that will be invoked when changing the page.
     * @remarks
     * This will be invoked when the drag animation begins or when the page changes
     * programatically. It can be used to co-ordinate with other behaviors.
     *
     * @param currentIndex - The current page number
     * @param previousIndex - The index of the previous page
     * @param pageComponent - The Page component for the current page.
     * @public
     */
    onChangePage(currentIndex: number, previousIndex: number, pageComponent: Page): void;
}

/**
 * All properties that can be used with the {@link Page} component it also extends all {@link ScrollProperties} properties.
 * @public
 */
export declare interface PageProperties {
    /**
     * Current swipe direction. Either "horizontal" or "vertical".
     */
    direction: PageDirection;
    /**
     * Width of the pages within the component. Either "auto" or "stretch" or a numeric value.
     */
    contentWidth: PageContentDimension | number;
    /**
     * Height of the pages within the component. Either "auto" or "stretch" or a numeric value.
     */
    contentHeight: PageContentDimension | number;
    /** Alignment of the pages within the component. Either "start", "center", or "end" */
    alignment: PageAlignment;
    /** Index of the current page. */
    currentPage: number;
    /**
     * @beta
     */
    animateCurrentPageUpdate: boolean;
    /** Gap between the page. */
    gap: number;
    /** Padding within the page component. */
    padding: number;
    /** Set padding on all sides, or one specific one. */
    paddingPerSide: boolean;
    /** Top padding within the page component. */
    paddingTop: number;
    /** Right padding within the page component. */
    paddingRight: number;
    /** Bottom padding within the page component. */
    paddingBottom: number;
    /** Left padding within the page component. */
    paddingLeft: number;
    /** Enable or disabling momentum, which defines if the page should auto-snap to the next page or not. */
    momentum: boolean;
    /** Enable dragging of the scroll inside the page component. */
    draggingEnabled: boolean;
    /** Pick an effect from one of the defaults. */
    defaultEffect: PageEffect;
    /** @beta */
    effect?: CustomPageEffect;
}

/** @public */
export declare interface PageProps extends PageProperties, FrameProperties, LayerProps, Partial<PageEvents>, Partial<DragEvents<typeof Frame>>, Partial<ScrollEvents> {
}

/**
 * @internal
 */
declare namespace PathSegment {
    type HandleMirroring = "straight" | "symmetric" | "disconnected" | "asymmetric";
}

/**
 * @internal
 */
declare class PathSegment extends PathSegmentRecord {
    x: number;
    y: number;
    handleMirroring: PathSegment.HandleMirroring;
    handleOutX: number;
    handleOutY: number;
    handleInX: number;
    handleInY: number;
    radius: number;
    toJS(): any;
    toJSON(): any;
}

/**
 * @internal
 */
declare namespace PathSegment {
    const point: (pathSegment: PathSegment) => {
        x: number;
        y: number;
    };
    const handleOut: (pathSegment: PathSegment) => {
        x: number;
        y: number;
    };
    const handleIn: (pathSegment: PathSegment) => {
        x: number;
        y: number;
    };
    const calculatedHandleOut: (pathSegment: PathSegment) => Point;
    const calculatedHandleIn: (pathSegment: PathSegment) => Point;
    const curveDefault: (points: PathSegment[], index: number) => Point;
}

/**
 * @internal
 */
declare const PathSegmentRecord: Record.Class;

/**
 * @public
 */
export declare function Point(x: number, y: number): Point;

/**
 * @public
 */
export declare interface Point {
    x: number;
    y: number;
}

/**
 * @public
 */
export declare namespace Point {
    /** @alpha */
    const add: (...args: Point[]) => Point;
    /** @alpha */
    const subtract: (a: Point, b: Point) => Point;
    /** @alpha */
    const multiply: (a: Point, b: number) => Point;
    /** @alpha */
    const divide: (a: Point, b: number) => Point;
    /** @alpha */
    const absolute: (point: Point) => Point;
    /** @internal */
    const reverse: (point: Point) => Point;
    /** @internal */
    const pixelAligned: (point: Point, offset?: Point) => Point;
    /** @alpha */
    const distance: (a: Point, b: Point) => number;
    /** @alpha */
    const angle: (a: Point, b: Point) => number;
    /** @public */
    const isEqual: (a: Point, b: Point) => boolean;
    /** @internal */
    const rotationNormalizer: () => (value: number) => number;
    /** @alpha */
    export function center(a: Point, b: Point): {
        x: number;
        y: number;
    };
}

/**
 * Prints to the console.
 *
 * @param args - Arguments to print
 * @public
 */
export declare function print(...args: any[]): void;

/** @public */
export declare type PropertyControls<ComponentProps = any, ArrayTypes = any> = {
    [K in keyof ComponentProps]?: ControlDescription<Partial<ComponentProps>, ArrayTypes>;
};

/**
 * @beta
 * @deprecated Use Data instead
 */
export declare function PropertyStore<T extends object = object>(initial?: Partial<T> | object, makeAnimatables?: boolean): AnimatableObject<T>;

/**
 * @beta
 * @deprecated Use Data instead
 */
export declare namespace PropertyStore {
    export function addObserver<T extends object>(target: T, observer: Observer<T>): Cancel;
}

/**
 * @internal
 */
declare type PropertyTree = {
    componentClass?: string;
    name?: string | null;
    children?: PropertyTree[];
    props?: any;
};

/**
 * @internal
 */
declare interface Props {
}

/**
 * @public
 */
declare interface RadialGradient {
    alpha: number;
    widthFactor: number;
    heightFactor: number;
    centerAnchorX: number;
    centerAnchorY: number;
    start: string;
    end: string;
}

/**
 * @public
 */
declare namespace RadialGradient {
    /**
     * @param radialGradient
     * @public
     */
    function isRadialGradient(radialGradient: any): radialGradient is RadialGradient;
    /** @alpha */
    function toCSS(radialGradient: RadialGradient): string;
}

declare interface RadiusProperties {
    radius: RadiusValue | Partial<{
        topLeft: RadiusValue;
        topRight: RadiusValue;
        bottomLeft: RadiusValue;
        bottomRight: RadiusValue;
    }>;
}

declare type RadiusValue = number | Animatable<number> | string;

/**
 * @internal
 */
declare type ReactComponentDefinition<P = any> = ComponentDefinition<P> & {
    class: React.ComponentType<P>;
};

/**
 * @public
 */
export declare interface Rect extends Point, Size {
}

/**
 * @public
 */
export declare namespace Rect {
    /**
     *
     * @param rect
     * @param other
     * @public
     */
    export function equals(rect: Rect | null, other: Rect | null): boolean;
    /** @alpha */
    const atOrigin: (size: Size) => {
        x: number;
        y: number;
        width: number;
        height: number;
    };
    /** @alpha */
    const fromTwoPoints: (a: Point, b: Point) => Rect;
    /** @alpha */
    const fromRect: (rect: ClientRect) => Rect;
    /** @alpha */
    const multiply: (rect: Rect, n: number) => Rect;
    /** @alpha */
    const divide: (rect: Rect, n: number) => Rect;
    /** @alpha */
    const offset: (rect: Rect, delta: Partial<Point>) => Rect;
    /** @alpha */
    export function inflate(rect: Rect, value: number): Rect;
    /** @alpha */
    const pixelAligned: (rect: Rect) => Rect;
    /** @alpha */
    const halfPixelAligned: (rect: Rect) => Rect;
    /** @alpha */
    const round: (rect: Rect, decimals?: number) => Rect;
    /** @alpha */
    const roundToOutside: (rect: Rect) => Rect;
    /**
     * @param rect
     * @beta
     */
    const minX: (rect: Rect) => number;
    /**
     * @param rect
     * @beta
     */
    const maxX: (rect: Rect) => number;
    /**
     * @param rect
     * @beta
     */
    const minY: (rect: Rect) => number;
    /**
     * @param rect
     * @beta
     */
    const maxY: (rect: Rect) => number;
    /** @internal */
    const positions: (rect: Rect) => {
        minX: number;
        midX: number;
        maxX: number;
        minY: number;
        midY: number;
        maxY: number;
    };
    /**
     *
     * @param rect
     * @beta
     */
    const center: (rect: Rect) => {
        x: number;
        y: number;
    };
    /** @internal */
    const fromPoints: (ps: Point[]) => {
        x: number;
        y: number;
        width: number;
        height: number;
    };
    /**
     * Returns a rect containing all input rects
     * @param rect a list of rectangels
     * @returns A rectangle that fits exactly around the input rects
     * @internal
     */
    const merge: (...rect: Rect[]) => Rect;
    /** @alpha */
    const intersection: (rect1: Rect, rect2: Rect) => Rect;
    /**
     * Returns all the corner points for a rect
     * @param rect
     * @internal
     */
    const points: (rect: Rect) => Point[];
    /**
     * Checks if a rectangle contains a point
     * @param rect The rectangle to check
     * @param point The point to check
     * @returns true if the provided rectangle contains the provided point
     * @beta
     */
    const containsPoint: (rect: Rect, point: Point) => boolean;
    /**
     * Returns wether a rect contains another rect entirely
     * @param rectA
     * @param rectB
     * @returns true if rectA contains rectB
     */
    const containsRect: (rectA: Rect, rectB: Rect) => boolean;
    /** @alpha */
    const toCSS: (rect: Rect) => {
        display: string;
        transform: string;
        width: string;
        height: string;
    };
    /** @alpha */
    const inset: (rect: Rect, n: number) => {
        x: number;
        y: number;
        width: number;
        height: number;
    };
    /** @alpha */
    const intersects: (rectA: Rect, rectB: Rect) => boolean;
    /** @internal */
    const overlapHorizontally: (rectA: Rect, rectB: Rect) => boolean;
    /** @internal */
    const overlapVertically: (rectA: Rect, rectB: Rect) => boolean;
    /** @internal */
    const doesNotIntersect: (rect: Rect, rects: Rect[]) => boolean;
    /**
     *
     * @param rectA
     * @param rectB
     * @return if the input rectangles are equal in size and position
     * @public
     */
    const isEqual: (rectA: Rect | null, rectB: Rect | null) => boolean;
    /** @internal */
    const cornerPoints: (rect: Rect) => Point[];
    /** @internal */
    const midPoints: (rect: Rect) => Point[];
    /** @internal */
    const pointDistance: (rect: Rect, point: Point) => number;
    /** @internal */
    const fromAny: (rect: any, defaults?: {
        x: number;
        y: number;
        width: number;
        height: number;
    }) => Rect;
}

/**
 * @internal
 */
declare interface RenderEnvironment {
    imageBaseURL: string;
    target: RenderTarget;
    zoom: number;
}

/**
 * @internal
 */
declare const RenderEnvironment: RenderEnvironment;

/**
 * @beta
 */
export declare enum RenderTarget {
    canvas = "CANVAS",
    export = "EXPORT",
    thumbnail = "THUMBNAIL",
    preview = "PREVIEW"
}

/**
 * This is a read-only equivalent of RenderEnvironment.target that is exposed to components for context-dependent rendering
 * @beta
 */
export declare namespace RenderTarget {
    export function current(): RenderTarget;
    export function hasRestrictions(): boolean;
}

/**
 * The Scroll component in Framer allows you create scrollable areas.
 * It can be imported from the Framer Library and used in code components.
 * Add children that exceed the height or width of the component to create
 * horizontally or vertically scrollable areas.
 * See {@link ScrollProperties} for its properties
 * @public
 */
export declare class Scroll extends React.Component<Partial<ScrollProps>> {
    /** @internal */
    static _isStylable: boolean;
    /** @internal */
    static supportsConstraints: boolean;
    /** @internal */
    static scrollProps: ScrollProperties;
    /** @internal */
    static defaultProps: ScrollProps;
    /** @internal */
    static propertyControls: PropertyControls<ScrollProps>;
    /** @internal */
    props: ScrollProps;
    private wrapHandlers;
    /** @internal */
    render(): JSX.Element;
}

declare type ScrollDirection = "horizontal" | "vertical" | "both";

declare type ScrollEventHandler = (event: FramerEvent, scrollComponent: Scroll) => void;

/**
 * @public
 */
export declare interface ScrollEvents {
    /** On start of scroll. */
    onScrollStart: ScrollEventHandler;
    /** On scroll. */
    onScroll: ScrollEventHandler;
    /** On end of scroll. */
    onScrollEnd: ScrollEventHandler;
    /** @beta */
    onScrollSessionStart: ScrollEventHandler;
    /** @beta */
    onScrollSessionEnd: ScrollEventHandler;
}

/**
 * The properties for the {@link Scroll} component, which are also available within other components, like {@link Page}.
 * @public
 */
export declare interface ScrollProperties {
    draggingEnabled: boolean;
    /** Scrolling direction. */
    direction: ScrollDirection;
    /** Lock the current scrolling direction. */
    directionLock: boolean;
    /**
     * @internalremarks
     * Ignored and deprecated; see https://github.com/framer/company/issues/10018 for future direction
     * @deprecated mouseWheel is always enabled
     */
    mouseWheel: boolean;
    /** Horizontal offset of the scrollable content. */
    contentOffsetX: number | Animatable<number> | null;
    /** Vertical offset of the scrollable content. */
    contentOffsetY: number | Animatable<number> | null;
}

/** @public */
export declare interface ScrollProps extends ScrollProperties, FrameProperties, LayerProps, Partial<ScrollEvents>, Partial<DragEvents<DraggableType>> {
}

declare interface SegmentedEnumControlDescription<P = any> extends BaseControlDescription<P> {
    type: ControlType.SegmentedEnum;
    defaultValue?: string;
    options: string[];
    optionTitles?: string[] | ((props: P | null) => string[]);
}

/**
 * @internal
 */
export declare function serverURL(...paths: string[]): string;

/**
 * This function sets the global render environment Framer Core uses to render.
 * Because it sets global state, there should be only one thing responsable for calling it in every react app (e.g. Vekter and Preview)
 * @internal
 */
export declare function setGlobalRenderEnvironment(environment: Partial<RenderEnvironment>): void;

declare interface Shadow {
    color: string;
    x: number;
    y: number;
    blur: number;
}

declare namespace Shadow {
    function is(shadow: any): shadow is Shadow;
}

/**
 * @public
 */
export declare function Size(width: number, height: number): Size;

/**
 * @public
 */
export declare interface Size {
    width: number;
    height: number;
}

/**
 * @public
 */
export declare namespace Size {
    /**
     * @param sizeA
     * @param sizeB
     * @alpha
     */
    const equals: (sizeA: Size | null, sizeB: Size | null) => boolean;
    /**
     *
     * @param fromSize The initial size
     * @param toSize The size to update to
     * @param keepAspectRatio If the updating should preserve the aspect ratio
     * @remark
     * keepAspectRatio only works if passing a toSize with only a width or height
     * @alpha
     */
    const update: (fromSize: Size, toSize: Partial<Size>, keepAspectRatio?: boolean) => {
        width: number;
        height: number;
    };
    /**
     *
     * @param sizeA
     * @param sizeB
     * @alpha
     */
    export function subtract(sizeA: Size, sizeB: Size): {
        width: number;
        height: number;
    };
    /**
     * @public
     */
    const zero: Size;
    /**
     * Checks if the size has a zero width and zero height
     * @param size size to check
     * @public
     */
    const isZero: (size: Size) => boolean;
    /**
     * @param width
     * @param height
     * @param size
     * @alpha
     */
    const defaultIfZero: (width: number, height: number, size: Size) => Size;
}

/**
 * Animator class using a spring curve
 * @beta
 */
export declare class SpringAnimator<Value> implements Animator<Value, SpringOptions> {
    private interpolation;
    private options;
    private current;
    private destination;
    private difference;
    private state;
    private integrator;
    private interpolator;
    constructor(options: Partial<SpringOptions>, interpolation: Interpolation<Value>);
    isReady(): boolean;
    next(delta: number): Value;
    isFinished(): boolean;
    setFrom(value: Value): void;
    setVelocity(velocity: number): void;
    progress(): number;
    setTo(value: Value): void;
    /** @internal */
    getState(): State;
    updateInterpolator(): void;
}

declare type SpringOptions = TensionFrictionSpringOptions | DampingDurationSpringOptions;

/**
 * @public
 */
export declare class Stack extends Layer<Partial<StackProperties>, StackState> {
    /** @internal */
    static _isStylable: boolean;
    /** @internal */
    static defaultStackSpecificProps: StackSpecificProps;
    /** @internal */
    static readonly defaultProps: StackProperties;
    /** @internal */
    props: StackProperties;
    /** @internal */
    static propertyControls: PropertyControls<StackProperties>;
    /** @internal */
    readonly rect: Rect;
    /** @internal */
    state: StackState;
    /** @internal */
    static getDerivedStateFromProps(nextProps: Partial<StackProperties>, prevState: StackState): StackState | null;
    /** @internal */
    render(): JSX.Element | null;
    /** @internal */
    static _cssExport(props: StackSpecificProps): React.CSSProperties;
    /** @internal */
    static paddingSize(props: StackProperties): Size;
    /** @internal */
    static childLayoutRects(childSizes: Size[], size: Size, props: StackProperties, invisibleItems: number[]): Rect[];
    /** @internal */
    static autoSize(minChildrenSizes: Size[], props: StackProperties, invisibleItems: number[]): {
        width: number;
        height: number;
    };
    /** @internal */
    static freeSpace(size: Size, autoSize: Size, childFractions: Size, props: StackProperties): WithFractionOfFreeSpace;
    private layoutChildren;
    private layoutPlaceholders;
    /**
     * @alpha
     */
    static childInsertion(childRects: Rect[], containerSize: Size, insertion: Point, insertionMaxSize: Size, properties: StackProperties): {
        index: number;
        line: Line;
    };
}

/**
 * @public
 */
declare type StackAlignment = "start" | "center" | "end";

/**
 * @public
 */
declare type StackDirection = "horizontal" | "vertical";

/**
 * @public
 */
declare type StackDistribution = "start" | "center" | "end" | "space-between" | "space-around" | "space-evenly";

/**
 * @beta
 */
declare interface StackPlaceHolders {
    index: number;
    sizes: Size[];
}

/**
 * @public
 */
declare interface StackProperties extends StackSpecificProps, FrameProperties, LayerProps {
}

/**
 * @public
 */
declare interface StackSpecificProps {
    direction: StackDirection;
    distribution: StackDistribution;
    alignment: StackAlignment;
    gap: number;
    padding: number;
    paddingPerSide: boolean;
    paddingTop: number;
    paddingRight: number;
    paddingLeft: number;
    paddingBottom: number;
    /** @internal */
    placeholders?: StackPlaceHolders;
}

/**
 * @public
 */
declare interface StackState {
    size: AnimatableObject<Size> | Size | null;
    shouldCheckImageAvailability: boolean;
    currentBackgroundImageSrc: string | null;
}

/**
 * @internal
 */
declare interface StackState_2 {
    current: number;
    previous: number;
    stack: NavigationItem[];
}

/**
 * @internal
 */
declare interface State {
    x: number;
    v: number;
}

/**
 * @internal
 */
declare interface State_2 {
    update: number;
}

declare interface StringControlDescription<P = any> extends BaseControlDescription<P> {
    type: ControlType.String;
    defaultValue?: string;
    placeholder?: string;
}

/**
 * @internal
 */
declare type StrokeAlignment = "center" | "inside";

/**
 * @internal
 */
export declare class SVG extends Layer<SVGProperties, {}> {
    static supportsConstraints: boolean;
    static defaultSVGProps: SVGProps;
    static readonly defaultProps: SVGProperties;
    static frame(props: Partial<ConstraintProperties>): Rect;
    readonly frame: Rect;
    private _constraintCache;
    constraintValues(): UserConstraintValues;
    componentDidUpdate(prevProps: SVGProperties): void;
    render(): JSX.Element | null;
}

/**
 * @internal
 */
declare interface SVGProperties extends SVGProps, LayerProps {
}

/**
 * @internal
 */
declare interface SVGProps extends ConstraintProperties, Partial<FilterProperties & BackgroundFilterProperties & RadiusProperties & WithOpacity> {
    rotation: Animatable<number> | number;
    visible: boolean;
    name?: string;
    fill?: Animatable<Background> | Background | null;
    svg: string;
    intrinsicWidth?: number;
    intrinsicHeight?: number;
    shadows: Shadow[];
}

declare interface TensionFrictionSpringOptions {
    tension: number;
    friction: number;
    tolerance: number;
    velocity: number;
}

/**
 * @internal
 */
export declare class Text extends Layer<TextProperties, {}> {
    static supportsConstraints: boolean;
    static defaultTextProps: TextProps;
    static readonly defaultProps: TextProperties;
    editorText: string | undefined;
    private editorState;
    readonly frame: Rect;
    private _constraintCache;
    constraintValues(): UserConstraintValues;
    private editorStateForContentState;
    componentWillReceiveProps(nextProps: TextProperties): void;
    render(): JSX.Element;
    /** Used by the ComponentContainerContext */
    private renderMain;
    private blockRendererFn;
    private onChange;
}

/**
 * @internal
 */
declare type TextAlignment = "left" | "right" | "center" | undefined;

declare interface TextColorProperties {
    color: Color | string;
}

/**
 * @internal
 */
declare interface TextProperties extends TextProps, LayerProps {
    rawHTML?: string;
    isEditable?: boolean;
    /** @internal for testing */
    environment?(): RenderTarget;
}

/**
 * @internal
 */
declare interface TextProps extends ConstraintProperties, Partial<FilterProperties> {
    rotation: Animatable<number> | number;
    visible: boolean;
    name?: string;
    contentState?: any;
    alignment: TextAlignment;
    autoSize: boolean;
    clip: boolean;
    size: Size;
    opacity?: number;
    shadows: Shadow[];
    style?: React.CSSProperties;
    text?: string;
    font?: string;
}

declare type ToAnimatableOrValue<PossiblyAnimatable> = PossiblyAnimatable extends Animatable<infer Value> ? Value | Animatable<Value> : PossiblyAnimatable | Animatable<PossiblyAnimatable>;

/**
 * @public
 */
declare type TransactionId = number;

/**
 * @public
 */
export declare function transform<InputValue, OutputValue>(from: [InputValue, InputValue], to: [OutputValue, OutputValue], options?: Partial<transform.Options<InputValue, OutputValue>>): (from: InputValue) => OutputValue;

/**
 * @public
 */
export declare namespace transform {
    export interface Options<InputValue, OutputValue> {
        inputInterpolation: Interpolation<InputValue>;
        outputInterpolation: Interpolation<OutputValue>;
        limit: boolean;
        colorModel: ColorMixModelType;
    }
    export function value<InputValue, OutputValue>(input: InputValue, from: [InputValue, InputValue], to: [OutputValue, OutputValue], options?: Partial<transform.Options<InputValue, OutputValue>>): OutputValue;
}

declare interface TransformProperties {
    z: Animatable<number> | number;
    rotation: Animatable<number> | number;
    rotationX: Animatable<number> | number;
    rotationY: Animatable<number> | number;
    rotationZ: Animatable<number> | number;
    scale: Animatable<number> | number;
    scaleX: Animatable<number> | number;
    scaleY: Animatable<number> | number;
    scaleZ: Animatable<number> | number;
    skew: Animatable<number> | number;
    skewX: Animatable<number> | number;
    skewY: Animatable<number> | number;
    originX: Animatable<number> | number;
    originY: Animatable<number> | number;
    originZ: Animatable<number> | number;
}

/**
 * @internal
 */
export declare function updateComponentLoader(script: string, framer: any): void;

/**
 * @public
 */
declare type UpdateFunction<Value> = (change: Change<Value>, transaction?: TransactionId) => void;

/**
 * @public
 */
declare interface UpdateObserver<Value> {
    onUpdate(handler: Observer<Value>): Cancel;
}

/**
 * @internal
 */
declare interface UserConstraintValues extends ConstraintValuesBase {
    width: number | null;
    height: number | null;
}

/**
 * @internal
 */
export declare class ValueInterpolation implements Interpolation {
    private options;
    /**
     * @internal
     */
    constructor(options?: Partial<InterpolationOptions>);
    /**
     * @internal
     */
    protected interPolationForValue(value: any): Interpolation;
    /**
     * @beta
     */
    interpolate: <T>(from: T, to: T) => (progress: number) => T;
    /**
     * @beta
     */
    difference: <T>(from: T, to: T) => number;
}

/**
 * @internal
 */
export declare class Vector extends Layer<VectorProperties, {}> {
    static defaultVectorProps: VectorProps;
    static readonly defaultProps: VectorProperties;
    render(): JSX.Element | null;
    renderElement(element: JSX.Element): JSX.Element;
}

/**
 * @internal
 */
export declare class VectorGroup extends Layer<VectorGroupProperties, {}> {
    static defaultVectorGroupProps: VectorGroupProps;
    static readonly defaultProps: VectorGroupProperties;
    render(): JSX.Element | null;
    renderElement(element: JSX.Element): JSX.Element;
}

/**
 * @alpha
 */
declare interface VectorGroupProperties extends VectorGroupProps, LayerProps {
}

/**
 * @alpha
 */
declare interface VectorGroupProps {
    name?: string;
    opacity?: number | string;
    visible: boolean;
    x: number;
    y: number;
    rotation: number;
    width: number;
    height: number;
    targetName?: string;
    defaultName: string;
    isRootVectorNode: boolean;
    frame: Rect;
    includeTransform?: boolean;
}

/**
 * @internal
 */
declare interface VectorProperties extends VectorProps, LayerProps {
}

/**
 * @internal
 */
declare interface VectorProps extends Partial<FillProperties> {
    isRootVectorNode: boolean;
    name: string | null;
    includeTransform?: boolean;
    defaultFillColor?: string;
    defaultStrokeColor?: string;
    defaultStrokeWidth?: number;
    defaultStrokeAlignment?: StrokeAlignment;
    width: number;
    height: number;
    rotation: number;
    frame: Rect;
    opacity?: number;
    calculatedPath: WithPath[];
    shapeId?: string;
    insideStroke: boolean;
    strokeEnabled: boolean;
    strokeClipId?: string;
    strokeWidth?: number;
    idAttribute?: string;
    shadows: BoxShadow[];
    rect: Rect;
    strokeAlpha: number;
    lineCap: LineCap;
    lineJoin: LineJoin;
    strokeColor: string;
    strokeMiterLimit: number;
    strokeDashArray: string;
    strokeDashOffset: number;
}

/**
 * This version is automatically updated by the Makefile
 * @public
 */
export declare const version = "0.10.10";

declare type VisualProperties = Partial<BackgroundProperties & RadiusProperties & FilterProperties & BackgroundFilterProperties & BlendingProperties & OverflowProperties & BoxShadowProperties & WithOpacity & TextColorProperties>;

declare interface WithEventsProperties extends WithPanHandlers, WithTapHandlers, WithMouseHandlers, WithMouseWheelHandler {
}

/**
 * @public
 */
declare interface WithFractionOfFreeSpace {
    /**
     * All free space in the parent, in px.
     * @internal
     */
    freeSpaceInParent: Size;
    /**
     * The sum of all "fr" values in siblings wishing to consume free space. Each free space consuming child must divide its own "fr" value by this value.
     * @internal
     */
    freeSpaceUnitDivisor: Size;
}

declare interface WithMouseHandlers {
    onMouseDown: EventHandler;
    onClick: EventHandler;
    onMouseUp: EventHandler;
    onMouseEnter: EventHandler;
    onMouseLeave: EventHandler;
}

declare interface WithMouseWheelHandler {
    onMouseWheelStart: EventHandler;
    onMouseWheel: EventHandler;
    onMouseWheelEnd: EventHandler;
}

/**
 * @internal
 */
export declare function WithNavigator<T, BaseProps extends React.ClassAttributes<T> & WithNavigator>(BaseComponent: React.ComponentType<BaseProps & {
    onTap?: any;
}>, navigationTransition: string, navigationTransitionDirection: NavigationTransitionDirection, navigationComponent: (() => React.ReactNode) | undefined, navigationTransitionOverrides?: NavigationTransitionOptions): React.ComponentClass<BaseProps & WithNavigator>;

/**
 * @internal
 */
export declare interface WithNavigator {
    navigator: null | Navigator;
}

declare interface WithOpacity {
    opacity: number | Animatable<number>;
}

/**
 * @internal
 */
export declare function WithOverride<T extends object>(Component: React.ComponentType<T>, override: Override<T>): (props: T) => JSX.Element;

/**
 * @internal
 */
declare interface WithPackage {
    package: string;
}

declare interface WithPanHandlers {
    onPanStart: EventHandler;
    onPan: EventHandler;
    onPanEnd: EventHandler;
}

/**
 * @internal
 */
declare interface WithPath {
    pathSegments: List<PathSegment>;
    pathClosed: boolean;
}

declare interface WithTapHandlers {
    onTapStart: EventHandler;
    onTap: EventHandler;
    onTapEnd: EventHandler;
}

export { }
